/**
 * FreqParameterSettings  07/14/2011
 * 
 * Sets the frequency ratios, number of voices, etc.
 *  for Floomptagorpt application
 *
 * Copyright (C) 2011  Thomas G.Digby (bubbles@plergb.com)
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see
 *
 *   <http://www.gnu.org/licenses/>.
 *
 */

package floomptagorpt;

/**
 *
 * @author bubbles
 */


import java.awt.Color;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JSeparator;


public class FreqParameterSettings {

    // This is a string of consecutive integers,
    // so we can use the array index to callculate the value
    static final String[]
        HOW_MANY_VOICES_STRINGS = {"1", "2", "3", "4", "5", "6", "7", "8"};
    static final int DEFAULT_HOW_MANY_VOICES_INDEX = 5;
    static final int DEFAULT_HOW_MANY_VOICES =
            DEFAULT_HOW_MANY_VOICES_INDEX + 1;

    // semitones between voices
    static final String[] FREQ_SEP_STRINGS = {
        " 1", " 2", " 3", " 4", " 5", " 6",
        " 7", " 8", " 9", "10", "11", "12", "13", "14", "15", "16",
        "17", "18", "19", "20"};
    static final int DEFAULT_FREQ_SEP_INDEX = 11;
    static final int DEFAULT_FREQ_SEP = DEFAULT_FREQ_SEP_INDEX + 1;

    // ratios of voics
    static final String[] FREQ_RATIO_STRINGS = {
        "3:1", "2.5:1", "2:1", "3:2", "4:3", "5:4", "6:5", "7:6", "8:7",
        "9:8", "10:9", "11:10", "12:11", "13:12", "14:13", "15:14", "16:15",
        "17:16", "18:17", "19:18", "20:19", "21:20"
    };
    static final double[] freqRatios = {
        3, 2.5, 2, 1.5, 4.0/3, 1.25, 1.2, 7.0/6, 8.0/7,
        9.0/8, 10.0/9, 1.1, 12.0/11, 13.0/12, 14.0/13, 15.0/14, 16.0/15,
        17.0/16, 18.0/17, 19.0/18, 20.0/19, 21.0/20
    };

    static final int DEFAULT_FREQ_RATIO_INDEX = 2;
    static final int DEFAULT_FREQ_RATIO = 2;
    
    static final int[] CENTER_FREQ_CHOICES = {
        55, 110, 220, 440, 880, 1760, 3520};
    static final int DEFAULT_CENTER_FREQ_CHOICE = 3; // index into array
    static final String[] CENTER_FREQ_STRINGS = {
        "  55Hz", " 110Hz", " 220Hz", " 440Hz",
        " 880Hz", "1760Hz", "3520Hz"};
    

    public int howManyVoices = DEFAULT_HOW_MANY_VOICES;
    int semiTonesBetweenVoices = DEFAULT_FREQ_SEP;
    public double ratioOfVoices = DEFAULT_FREQ_RATIO;
    int defaultFreqRatioIndex;

    int centerFreqChoice = DEFAULT_CENTER_FREQ_CHOICE;
    int centerFreq = CENTER_FREQ_CHOICES[DEFAULT_CENTER_FREQ_CHOICE];

    // calculated at run time
    public double freqSpanRatio, halfSpanRatio, minFreq, maxFreq;
    int lastSelectedRatioIndex;

    JPanel freqPane;

    // no need for event listeners, since these are polled
    JComboBox numVoicesBox, freqSepBox, centerFreqBox;

    boolean rationalFreqRatios = false;


    FreqParameterSettings(boolean rationalRatios){

        rationalFreqRatios = rationalRatios;

        freqPane = new JPanel();
        freqPane.setLayout(
                new BoxLayout(freqPane, BoxLayout.PAGE_AXIS));
        freqPane.setBorder(
            BorderFactory.createLineBorder(Color.black));

        JLabel freqLabel = new JLabel("Frequecy Settings");
        freqPane.add(freqLabel);
        freqPane.add(new JSeparator());
        freqPane.add(new JSeparator());

        JLabel voiceLabel = new JLabel("Number of Voices");
        freqPane.add(voiceLabel);

        numVoicesBox = new JComboBox(HOW_MANY_VOICES_STRINGS);
        freqPane.add(numVoicesBox);
        numVoicesBox.setSelectedIndex(DEFAULT_HOW_MANY_VOICES_INDEX);

        freqPane.add(new JSeparator());

        JLabel freqSepLabel;
        if(rationalFreqRatios){ // integer ratios
            freqSepLabel = new JLabel("Frequency Ratio Between Voices");
            freqSepBox = new JComboBox(FREQ_RATIO_STRINGS);
            defaultFreqRatioIndex = (DEFAULT_FREQ_RATIO_INDEX);
        } else { // semitones
            freqSepLabel = new JLabel("Semitones Between Voices");
            freqSepBox = new JComboBox(FREQ_SEP_STRINGS);
            defaultFreqRatioIndex = (DEFAULT_FREQ_SEP_INDEX);
        } // semitones
        freqSepBox.setSelectedIndex(defaultFreqRatioIndex);
        lastSelectedRatioIndex = defaultFreqRatioIndex;
        freqPane.add(freqSepLabel);
        freqPane.add(freqSepBox);

        freqPane.add(new JSeparator());

        JLabel centerFreqLabel = new JLabel("Center Frequency");
        freqPane.add(centerFreqLabel);

        centerFreqBox = new JComboBox(CENTER_FREQ_STRINGS);
        freqPane.add(centerFreqBox);
        centerFreqBox.setSelectedIndex(DEFAULT_CENTER_FREQ_CHOICE);
        
    } // FreqParameterSettings()

    
    public void enableChanges(){
        
        numVoicesBox.setEnabled(true);
        freqSepBox.setEnabled(true);
        centerFreqBox.setEnabled(true);
        
    } // enableChanges()

    
    public void disableChanges(){

        numVoicesBox.setEnabled(false);
        freqSepBox.setEnabled(false);
        centerFreqBox.setEnabled(false);

    } // enableChanges()


    public void setDefault(){ // reset controls, etc., to default

        howManyVoices = DEFAULT_HOW_MANY_VOICES;
        semiTonesBetweenVoices = DEFAULT_FREQ_SEP;
        ratioOfVoices = DEFAULT_FREQ_RATIO;
        centerFreqChoice = DEFAULT_CENTER_FREQ_CHOICE;
        centerFreq = CENTER_FREQ_CHOICES[DEFAULT_CENTER_FREQ_CHOICE];
        numVoicesBox.setSelectedIndex(DEFAULT_HOW_MANY_VOICES_INDEX);
        freqSepBox.setSelectedIndex(defaultFreqRatioIndex);
        lastSelectedRatioIndex = defaultFreqRatioIndex;
        centerFreqBox.setSelectedIndex(DEFAULT_CENTER_FREQ_CHOICE);

        recalcFrequencies(); // recalculate frequency ratios, limits, etc.
        
    } // setDefault()


    public void cancelChanges(){ // reset controls to previous value

        numVoicesBox.setSelectedIndex(howManyVoices - 1);
        freqSepBox.setSelectedIndex(lastSelectedRatioIndex);
        centerFreqBox.setSelectedIndex(centerFreqChoice);

        // don't need to  recalculate frequency ratios, etc.

    } // cancelChanges()


    public void applyChanges(){ // get and use state of controls

        howManyVoices = numVoicesBox.getSelectedIndex() + 1;
        lastSelectedRatioIndex = freqSepBox.getSelectedIndex();

        if(rationalFreqRatios){ // integer ratios
            ratioOfVoices = freqRatios[lastSelectedRatioIndex];
        } else { // semitones

            semiTonesBetweenVoices = lastSelectedRatioIndex + 1;

            ratioOfVoices = Math.pow(2, (double)semiTonesBetweenVoices/12);

        } // semitones

        centerFreqChoice = centerFreqBox.getSelectedIndex();
        centerFreq = CENTER_FREQ_CHOICES[centerFreqChoice];

        recalcFrequencies(); // recalculate frequency ratios, limits, etc.
        
    } // applyChanges()


    void recalcFrequencies(){ // recalculate frequency ratios, limits, etc.
        
        freqSpanRatio = Math.pow(ratioOfVoices, howManyVoices);
        halfSpanRatio = Math.pow(freqSpanRatio, 0.5);

        minFreq = centerFreq / halfSpanRatio;
//        maxFreq = centerFreq * halfSpanRatio;
        maxFreq = minFreq * freqSpanRatio;
        
    } // recalcFrequencies()



    public String reportStatus(){ // tell user what has been selected

        String headerLine = "Frequency Settings\n";

        String voicesLine;
        if(rationalFreqRatios){  // expressed as frequency ratios
            voicesLine = "   " + howManyVoices
               + " Voices, with a frequecy ratio of "
               + ratioOfVoices + " to 1 \n";
        } else { // expresssed in semitones
            voicesLine = "   " + howManyVoices + " Voices, Spaced "
               + semiTonesBetweenVoices + " semitones apart\n";
        } // else

        String freqLine = "   between " + (int) minFreq + "Hz and "
               + (int) maxFreq + "Hz, centered on " + centerFreq + " Hz\n";

        return headerLine + voicesLine + freqLine;

    } // reportStatus()

} // class FreqParameterSettings
