/**
 * MainPanel  07/14/2011
 *
 * Main panel for Floomptagorpt application
 *
 * Copyright (C) 2011  Thomas G.Digby (bubbles@plergb.com)
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see
 *
 *   <http://www.gnu.org/licenses/>.
 *
 */


package floomptagorpt;

/**
 *
 * @author bubbles
 */

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingWorker;



public class MainPanel extends JPanel
    implements ActionListener, ChangeListener{

//    static final boolean VERBOSE = true; // for debugging
      static final boolean VERBOSE = false; // for uncluttered display
    
    TopLevelControls topLevelControls;
    HardwareManager engine;
    WaveFormSettings waveFormSettings;
    FreqParameterSettings freqParameterSettings;
    MiscSettings miscSettings;

    // which mode for frequency ratios, semitones or rational fractions
    public static boolean rationalRatios = false;   // default to semitones
//    public static boolean rationalRatios = true;  // default to rational
    boolean lineReady = false;
//    volatile double newSweepRate = TopLevelControls.SWEEP_RATE_INIT;

    // depends on TopLevelControls.SWEEP_PROXY_INIT and sweep rate slider power
    // if proxy = 20, linear wants 2, square wants 0.4, cube wasnts 0.08
    volatile double newSweepRate = 0;
    static double startingSweepRate = 0.08;

    // what power law to use with sweep rate slider
    static final int LIN_LAW = 0;
    static final int SQR_LAW = 1;
    static final int CUBE_LAW = 2;
    static final int DEFAULT_LAW = CUBE_LAW;
    static int whatSliderLaw = DEFAULT_LAW;
    static boolean sliderLawSet = false;

    public volatile int volumeSetting = TopLevelControls.VOLUME_INIT;


    private EngineTask engineTask;


    public MainPanel() { // constructor
        super(new GridBagLayout());

        topLevelControls =
                new TopLevelControls(this, this, this);
        GridBagConstraints cRunPanel = new GridBagConstraints();
        cRunPanel.gridx = 0;
        cRunPanel.gridy = 0;
        cRunPanel.gridwidth = 2;
        add(topLevelControls.runPane, cRunPanel);

        waveFormSettings = new WaveFormSettings();
        GridBagConstraints cWFPanel = new GridBagConstraints();
        cWFPanel.gridx = 0;
        cWFPanel.gridy = 1;
        cWFPanel.anchor = GridBagConstraints.CENTER;
        add(waveFormSettings.waveformPane, cWFPanel);

        freqParameterSettings = new FreqParameterSettings(rationalRatios);
        GridBagConstraints cFreqPanel = new GridBagConstraints();
        cFreqPanel.gridx = 1;
        cFreqPanel.gridy = 1;
        cFreqPanel.anchor = GridBagConstraints.CENTER;
        add(freqParameterSettings.freqPane, cFreqPanel);

        miscSettings = new MiscSettings();
        GridBagConstraints cMiscPanel = new GridBagConstraints();
        cMiscPanel.gridx = 2;
        cMiscPanel.gridy = 1;
        cMiscPanel.anchor = GridBagConstraints.CENTER;
        add(miscSettings.miscPane, cMiscPanel);        

        GridBagConstraints cSetButnPanel = new GridBagConstraints();
        cSetButnPanel.gridx = 0;
        cSetButnPanel.gridy = 2;
        cSetButnPanel.gridwidth = 3;
        cSetButnPanel.anchor = GridBagConstraints.CENTER;
        add(topLevelControls.setButtonPanel, cSetButnPanel);


        // Set up the audio line
        engine = new HardwareManager();

        topLevelControls.setOtherComponents(
                waveFormSettings, freqParameterSettings, miscSettings);

        waveFormSettings.setDefault();
        freqParameterSettings.setDefault();
        miscSettings.setDefault();

    } // MainPanel()  constructor


    public void tellUser(String theMessage){
        topLevelControls.tellUser(theMessage);
    } // tellUser()


    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();

        int toneAction = topLevelControls.checkButtons(command);

        if(VERBOSE) tellUser("Action wanted = " + toneAction + "\n");

        switch (toneAction){
            case TopLevelControls.PLAY_TONE:
                if(VERBOSE) tellUser("Starting Tone Engine \n");
                // see if line is open
                // open it if needed
                if(!lineReady){  // need to set up hardware
                    tellUser("Setting up Audio Line\n");
                    String audioFate = engine.setupAudioLines();
                    if (null == audioFate){
                        tellUser("Had a problem setting up the audio.");
                        } else {
                            tellUser(audioFate);
                        } // else
                    lineReady = true;
                } // need to set up hardware

                // once line is open
                // see if thread exists
                // if not, create it
                if(null == engineTask){
                    (engineTask = new EngineTask()).execute();
                } // if null

                // now start feeding samples
                engine.setWantRun();
                break;
            
            case TopLevelControls.STOP_TONE:
                if(VERBOSE) tellUser("Stopping Tone Engine \n");
                // stop feeding samples
                engine.setWantPause();
                // flush the buffer
                engine.flushLineBuffer();

                // kill the thread if it exists
                if(null != engineTask){
                    engineTask.cancel(true);
                    engineTask = null;
                } // !null

                break;

            case TopLevelControls.PAUSE_TONE:
                if(VERBOSE) tellUser("Pausing Tone Engine \n");
                engine.setWantPause();
                // may be restarted by Play commmand

                break;

            default: if(VERBOSE) tellUser("No Change to Tone Engine State \n");

        } // switch

    } // actionPerformed


    public void stateChanged(ChangeEvent e) {

        int sweepRateProxy;

        JSlider source = (JSlider)e.getSource();
//        sweepRateProxy = (int)source.getValue();

        if(topLevelControls.isSweepRate(e)) { // setting sweep rate

            sweepRateProxy = (int)source.getValue();
            switch(whatSliderLaw){

                case LIN_LAW: {
                    newSweepRate = convertFremSliderLinear(
                    sweepRateProxy, TopLevelControls.SWEEP_MAX,
                    Voices.MAX_SWEEP_RATE);
                    break;
                }

                case SQR_LAW: {
                    newSweepRate = convertFremSliderSquare(
                    sweepRateProxy, TopLevelControls.SWEEP_MAX,
                    Voices.MAX_SWEEP_RATE);
                    break;
                }

                case CUBE_LAW:
                default:
                {
                    newSweepRate = convertFremSliderCube(
                    sweepRateProxy, TopLevelControls.SWEEP_MAX,
                    Voices.MAX_SWEEP_RATE);
                    break;
                }

            } // switch

            if (!source.getValueIsAdjusting())
                tellUser("Sweep Rate changed to " +
                    sweepRateProxy + " arbitrary units. " + "\n"
                    + "or about " + newSweepRate +
                    " octaves per second. \n\n");

            if((null != engine) && (null != engine.myVoices))
                engine.myVoices.setSweepRate(newSweepRate);
            //  else tellUser("Sweep rate change ignored. \n\n");

        } // if isSweepRate
        else if(topLevelControls.isVolumeKnob(e)) { // setting volume

            volumeSetting = (int)source.getValue();
            engine.myVoices.setOutputLevel(volumeSetting);

            if (!source.getValueIsAdjusting())
                tellUser("Volume changed to " + 
                        volumeSetting + " percent \n\n");

       } // is isVolumeKnob

    } // stateChanged


    // possible ways to conert from int slider value to double sweep rate
    // linear, square law, cube law
    // may add a user setting to choose one

    double convertFremSliderLinear(
            int sliderValue, int sliderMax, double rateMax){

        return(rateMax * ((double)sliderValue/(double)sliderMax));

    } // convertFremSliderLinear()


    double convertFremSliderSquare(
            int sliderValue, int sliderMax, double rateMax){

        return(rateMax * 
                (double)(sliderValue * Math.abs(sliderValue)) /
                ((double)(sliderMax * sliderMax)));

    } // convertFremSliderSquare()


    double convertFremSliderCube(
            int sliderValue, int sliderMax, double rateMax){

        return(rateMax *
                ((double)(sliderValue * sliderValue * sliderValue)/
                (double)(sliderMax * sliderMax * sliderMax)));

    } // convertFremSliderCube()


    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private static void createAndShowGUI() {
        //Create and set up the window.
        JFrame frame = new JFrame("Continuous Rising or Falling Tone");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Create and set up the control pane.

        MainPanel generatorControlPane =
                new MainPanel();
        generatorControlPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(generatorControlPane);
        
        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }  // createAndShowGUI()

    
    /**
     * @param args the command line arguments
     */

    // called when run as an application
    // not called when run as part of an applet
    public static void main(String[] args) {
        // check for command-line argument
        for (String s: args) {
            if(s.equalsIgnoreCase("rational")) rationalRatios = true;
            if(s.equalsIgnoreCase("semitones")) rationalRatios = false;

            if(s.equalsIgnoreCase("linearlaw")){
                whatSliderLaw = LIN_LAW;
                startingSweepRate = 2;
            } // linear
            if(s.equalsIgnoreCase("squarelaw")){
                whatSliderLaw = SQR_LAW;
                startingSweepRate = 0.4;
            } // square
            if(s.equalsIgnoreCase("cubelaw")){
                whatSliderLaw = CUBE_LAW;
                startingSweepRate = 0.08;
            } // cube
        } // for

        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            } // run
        });

    } // main




    private class EngineTask extends SwingWorker<Void, Void> {
        @Override
        protected Void doInBackground() {
            engine.setUpForSamples(waveFormSettings.waveFormWanted,
                    freqParameterSettings.howManyVoices,
                    (float) freqParameterSettings.ratioOfVoices,
                    freqParameterSettings.minFreq,
                    miscSettings.softStart);

            if (!sliderLawSet){
                newSweepRate = startingSweepRate;
                sliderLawSet = true;
            }
            engine.myVoices.setSweepRate(newSweepRate);
            engine.myVoices.setOutputLevel(volumeSetting);

            while (!isCancelled()) {
                try {engine.generateNextSample();
                } //try
                catch (InterruptedException e) {
                    if(VERBOSE) tellUser(
                            "Tone Generation Thread interrupted. \n\n");
                } // catch

            } // while not canceled
            
            if(VERBOSE)
                tellUser("\n About to finish Generator Thread Run method \n\n");

            return null;

        } // doInBackground()

    } // class engineTask


} // class MainPanel
