/**
 * TopLevelControls  07/14/2011
 *
 * Main Floomptagorpt application buttons and frequency slider
 *
 * Copyright (C) 2011  Thomas G.Digby (bubbles@plergb.com)
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see
 *
 *   <http://www.gnu.org/licenses/>.
 *
 */


package floomptagorpt;

/**
 *
 * @author bubbles
 */

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;


public class TopLevelControls {

    // set in main panel setup
//    static final double SWEEP_RATE_INIT = 0.1;  // initial octaves per second

    // sweep rate slider values
    // arbitrary units for now
    static final int SWEEP_MIN = -100;
    static final int SWEEP_MAX =  100;
    static final int SWEEP_PROXY_INIT =  20;

    static final int VOLUME_MIN  =    0;
    static final int VOLUME_MAX  =  100;
//    static final int VOLUME_MAX  =  Voices.OUT_LEVEL_DIVISOR;
//    these are equal only for linear taper
    static final int VOLUME_INIT =   70;

    

    // button strings
    private static String RUN_COMMAND = "runCmd";
    private static String PAUSE_COMMAND = "pauseCmd";
    private static String STOP_COMMAND = "stopCmd";

    private static String DEFAULTS_COMMAND = "defaultsCmd";
    private static String CANCEL_COMMAND = "cancelCmd";
    private static String APPLY_COMMAND = "applyCmd";

    public static final int UNCH_TONE  = 0;
    public static final int STOP_TONE  = 1;
    public static final int PAUSE_TONE = 2;
    public static final int PLAY_TONE  = 3;

    JTextArea textArea;
    JLabel volumeLabel, sessionLogLabel, sweepRateLabel;
    JButton runButton, pauseButton, stopButton;
    JButton defaultsButton, cancelButton, applyButton;
    JPanel runPane;
    JPanel setButtonPanel;

    MainPanel myMainPanel;
    WaveFormSettings myWaveFormSettings;
    FreqParameterSettings myFreqParameterSettings;
    MiscSettings myMiscSettings;

    JSlider sweepRate, volumeKnob;

    
    public TopLevelControls(MainPanel myPanel,
            ActionListener myActionListener,
            ChangeListener myChangeListener) { // constructor

        myMainPanel = myPanel;
        
        
        //Create the components.

        volumeLabel = new JLabel("Output Level, % of Max", JLabel.LEFT);
        sessionLogLabel = new JLabel("Session Log", JLabel.CENTER);
        sweepRateLabel = new JLabel("Sweep Rate", JLabel.RIGHT);

        runButton = new JButton("Run");
        runButton.setActionCommand(RUN_COMMAND);
        runButton.addActionListener(myActionListener);

        pauseButton = new JButton("Pause");
        pauseButton.setActionCommand(PAUSE_COMMAND);
        pauseButton.addActionListener(myActionListener);

        stopButton = new JButton("Stop");
        stopButton.setActionCommand(STOP_COMMAND);
        stopButton.addActionListener(myActionListener);


        defaultsButton = new JButton("Defaults");
        defaultsButton.setActionCommand(DEFAULTS_COMMAND);
        defaultsButton.addActionListener(myActionListener);

        cancelButton = new JButton("Cancel");
        cancelButton.setActionCommand(CANCEL_COMMAND);
        cancelButton.addActionListener(myActionListener);

        applyButton = new JButton("Apply");
        applyButton.setActionCommand(APPLY_COMMAND);
        applyButton.addActionListener(myActionListener);


        sweepRate = new JSlider(JSlider.VERTICAL,
                                      SWEEP_MIN, SWEEP_MAX, SWEEP_PROXY_INIT);
        sweepRate.setMajorTickSpacing(SWEEP_MAX/5);
        sweepRate.setPaintTicks(true);
        sweepRate.setPaintLabels(true);
        sweepRate.addChangeListener(myChangeListener);


        volumeKnob = new JSlider(JSlider.VERTICAL,
                VOLUME_MIN, VOLUME_MAX, VOLUME_INIT);
        volumeKnob.setMajorTickSpacing(VOLUME_MAX/10);
        volumeKnob.setPaintTicks(true);
        volumeKnob.setPaintLabels(true);
        volumeKnob.addChangeListener(myChangeListener);


        textArea = new JTextArea(5, 30);
        textArea.setEditable(false);

        textArea.append(AboutInfo.COPYRIGHT_DATE);
        textArea.append(AboutInfo.FREE_SOFTWARE);
        textArea.append(AboutInfo.NO_WARRANTY);
        textArea.append(AboutInfo.SHOULD_HAVE_COPY);

        textArea.append("\n\nThis is the Session Log:  ");
        textArea.append("Status messages and such go here.\n\n");
        JScrollPane scrollPane = new JScrollPane(textArea);
        textArea.setLineWrap(true);
        textArea.setWrapStyleWord(true);


        //Lay everything out.

        JPanel labelPanel = new JPanel(new GridLayout(0,3));
        labelPanel.add(volumeLabel);
        labelPanel.add(sessionLogLabel);
        labelPanel.add(sweepRateLabel);

        JPanel runButtonPanel = new JPanel(new GridLayout(0,3));
        runButtonPanel.add(runButton);
        runButtonPanel.add(pauseButton);
        runButtonPanel.add(stopButton);
        runButton.setEnabled(true);
        pauseButton.setEnabled(false);
        stopButton.setEnabled(false);

        setButtonPanel = new JPanel(new GridLayout(0,3));
        setButtonPanel.add(defaultsButton);
        setButtonPanel.add(cancelButton);
        setButtonPanel.add(applyButton);
        defaultsButton.setEnabled(true);
        cancelButton.setEnabled(true);
        applyButton.setEnabled(true);

        runPane = new JPanel(new BorderLayout());
        runPane.setPreferredSize(new Dimension(800, 400));
	runPane.add(labelPanel, BorderLayout.PAGE_START);
        runPane.add(volumeKnob, BorderLayout.LINE_START);
        runPane.add(sweepRate, BorderLayout.LINE_END);
        runPane.add(scrollPane, BorderLayout.CENTER);
	runPane.add(runButtonPanel, BorderLayout.PAGE_END);
        // setButtonPanel is added to something in main panel

        
    } // TopLevelControls constructor


    public void setOtherComponents(WaveFormSettings wfSettings,
    FreqParameterSettings fpSettings, MiscSettings msSettings){

        myWaveFormSettings = wfSettings;
        myFreqParameterSettings = fpSettings;
        myMiscSettings = msSettings;

    } // setOtherComponents()


    public int checkButtons(String command){

        int toneAction = UNCH_TONE;
        // Cancel, Apply, Default don't directly control tone generator

        if (RUN_COMMAND.equals(command)) {
            // Run button clicked
            // Show message, then start engine
            runButton.setEnabled(false);
            pauseButton.setEnabled(true);
            stopButton.setEnabled(true);

            defaultsButton.setEnabled(false);
            cancelButton.setEnabled(false);
            applyButton.setEnabled(false);

            myWaveFormSettings.disableChanges();
            myFreqParameterSettings.disableChanges();
            myMiscSettings.disableChanges();

            toneAction = PLAY_TONE;
            tellUser("Running\n");

  
        } else if (PAUSE_COMMAND.equals(command)) {
            // Pause button clicked
            // Stop engine, then show message
            runButton.setEnabled(true);
            pauseButton.setEnabled(false);
            toneAction = PAUSE_TONE;
            tellUser("Paused\n");

        } else if (STOP_COMMAND.equals(command)) {
            // Stop button clicked
            // Stop engine, then show message
            runButton.setEnabled(true);
            pauseButton.setEnabled(false);
            stopButton.setEnabled(false);
            defaultsButton.setEnabled(true);
            cancelButton.setEnabled(true);
            applyButton.setEnabled(true);
            myWaveFormSettings.enableChanges();
            myFreqParameterSettings.enableChanges();
            myMiscSettings.enableChanges();
            toneAction = STOP_TONE;
            tellUser("Stopped\n\n" +
                "You may now change the settings \n\n");

        } else if (DEFAULTS_COMMAND.equals(command)) {
            // Defaults button clicked
            // Restore default settings
            myWaveFormSettings.setDefault();
            myFreqParameterSettings.setDefault();
            myMiscSettings.setDefault();

           tellUser("Returning to Default Settings: \n");
           tellUser(myWaveFormSettings.reportStatus());
           tellUser("\n");
           tellUser(myFreqParameterSettings.reportStatus());
           tellUser("\n");
           tellUser(myMiscSettings.reportStatus());
           tellUser("\n");

        } else if (CANCEL_COMMAND.equals(command)) {
            // Cancel button clicked
            // Revert controls to active settings
            myWaveFormSettings.cancelChanges();
            myFreqParameterSettings.cancelChanges();
            myMiscSettings.cancelChanges();

           tellUser("Canceling Changes, Retaining Existing Settings: \n");
           tellUser(myWaveFormSettings.reportStatus());
           tellUser("\n");
           tellUser(myFreqParameterSettings.reportStatus());
           tellUser("\n");
           tellUser(myMiscSettings.reportStatus());
           tellUser("\n");

        } else if (APPLY_COMMAND.equals(command)) {
            // Apply button clicked
            // Apply settings from controls
            myWaveFormSettings.applyChanges();
            myFreqParameterSettings.applyChanges();
            myMiscSettings.applyChanges();

           tellUser("New Settings are being applied: \n");
           tellUser(myWaveFormSettings.reportStatus());
           tellUser("\n");
           tellUser(myFreqParameterSettings.reportStatus());
           tellUser("\n");
           tellUser(myMiscSettings.reportStatus());
           tellUser("\n");

        } // end of else chain

        return toneAction;

    } // checkButtons()


    public boolean isSweepRate(ChangeEvent e){
        JSlider source = (JSlider)e.getSource();
        return(source == sweepRate);
    } // isSweepRate

    public boolean isVolumeKnob(ChangeEvent e){
        JSlider source = (JSlider)e.getSource();
        return(source == volumeKnob);
    } // isVolumeKnob

    
    public void tellUser(String theMessage){
        textArea.append(theMessage);
    } // tellUser()
    

} // class TopLevelControls
