/**
 * WaveFormSettings  07/14/2011
 *
 * Sets the waveform to use in Floomptagorpt application
 *
 * Copyright (C) 2011  Thomas G.Digby (bubbles@plergb.com)
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see
 *
 *   <http://www.gnu.org/licenses/>.
 *
 */

package floomptagorpt;

/**
 *
 * @author bubbles
 */

import java.awt.Color;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class WaveFormSettings {

    // constants for waveform selection
    public static final int WANT_SINE   = 0; // Sine Wave
    public static final int WANT_SAWTH  = 1; // Sawtooth Wave
    public static final int WANT_SQUARE = 2; // Square Wave
    public static final int WANT_TRIANG = 3; // Triangle Wave
    public static final int WANT_CUSTOM = 4; // Custom (for future use)
    
    public int waveFormWanted = WANT_SINE;
    
    String sinwString = "Sine";
    String sawtString = "Sawtooth";
    String sqString = "Square";
    String triangString = "Triangle";

    JPanel waveformPane;

    JRadioButton sineButton, sawtButton, sqButton, triangButton;


    public WaveFormSettings(){


        waveformPane = new JPanel();
        waveformPane.setLayout(
                new BoxLayout(waveformPane, BoxLayout.PAGE_AXIS));

        waveformPane.setBorder(
            BorderFactory.createLineBorder(Color.black));

        JLabel wfLabel = new JLabel("Waveform");
        waveformPane.add(wfLabel);

        sineButton = new JRadioButton(sinwString);
        sineButton.setActionCommand(sinwString);
        sineButton.setSelected(true);

        sawtButton = new JRadioButton(sawtString);
        sawtButton.setActionCommand(sawtString);

        sqButton = new JRadioButton(sqString);
        sqButton.setActionCommand(sqString);

        triangButton = new JRadioButton(triangString);
        triangButton.setActionCommand(triangString);

        ButtonGroup waveGroup = new ButtonGroup();
        waveGroup.add(sineButton);
        waveGroup.add(sawtButton);
        waveGroup.add(sqButton);
        waveGroup.add(triangButton);

        waveformPane.add(sineButton);
        waveformPane.add(sawtButton);
        waveformPane.add(sqButton);
        waveformPane.add(triangButton);

    } // WaveFormSettings()


    public void enableChanges(){

        sineButton.setEnabled(true);
        sawtButton.setEnabled(true);
        sqButton.setEnabled(true);
        triangButton.setEnabled(true);

    }// enableChanges()


    public void disableChanges(){

        sineButton.setEnabled(false);
        sawtButton.setEnabled(false);
        sqButton.setEnabled(false);
        triangButton.setEnabled(false);

    }// disableChanges()


    public void setDefault(){ // reset radio buttons, etc., to default

        waveFormWanted = WANT_SINE;

        triangButton.setSelected(false);
        sawtButton.setSelected(false);
        sqButton.setSelected(false);

        sineButton.setSelected(true);

    } // setDefault()


    public void cancelChanges(){ // reset radio buttons to previous value

        triangButton.setSelected(false);
        sawtButton.setSelected(false);
        sqButton.setSelected(false);
        sineButton.setSelected(false);
        
        if (WANT_TRIANG == waveFormWanted){
            triangButton.setSelected(true);
        } else if (WANT_SQUARE == waveFormWanted){
            sqButton.setSelected(true);
        } else if (WANT_SAWTH == waveFormWanted){
            sawtButton.setSelected(true);
        } else // only valid thing left is sine
            sineButton.setSelected(true);

    } // cancelChanges()


    public void applyChanges(){ // get and use state of radio buttons

        if(triangButton.isSelected()){
            waveFormWanted = WANT_TRIANG;
        } else if(sqButton.isSelected()){
            waveFormWanted = WANT_SQUARE;
        } else if(sawtButton.isSelected()){
            waveFormWanted = WANT_SAWTH;
        } else // only valid thing left is sine
            waveFormWanted = WANT_SINE;

    } // applyChanges()


    public String reportStatus(){ // tell user what has been selected

        if (WANT_TRIANG == waveFormWanted){
            return "Triangle Wave is selected. \n";
        } else if (WANT_SQUARE == waveFormWanted){
            return "Square Wave is selected. \n";
        } else if (WANT_SAWTH == waveFormWanted){
            return "Sawtooth Wave is selected. \n";
        } else // only valid thing left is sine
            return "Sine Wave is selected. \n";
        
    } // reportStatus()

}  // class WaveFormSettings
